/*
 * Photo Image Print System
 * Copyright (C) 2001-2005 EPSON AVASYS Corporation.
 * Copyright (C) Seiko Epson Corporation 2001-2008.
 *
 * This file is part of the `ekpd' program.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, EPSON AVASYS Corporation gives permission
 * to link the code of this program with the `cbt' library and
 * distribute linked combinations including the two.  You must obey
 * the GNU General Public License in all respects for all of the
 * code used other then `cbt'.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include "cbtd.h"

/* initialize system */
int
parameter_setup (P_CBTD_INFO p_info)
{
    char line [1024];    
    char name[255];
    char *def_name, *value = NULL;
    int i;
    FILE* file;
    char rc_file[] = PIPS_EKPD_RC_PATH;

    file = fopen (rc_file, "rt");
    if (file == NULL)
      return 1;

    /* Get default printer name */
    while (!feof(file))
      {
	fgets (line, 1024, file);
	if (!strstr (line, "defaultprinter"))
	  continue;	
	def_name = strchr (line, '>') + 1;
	i = strchr (def_name, '<') - def_name;
	def_name[i] = '\0';
	/* store the default name */
	strcpy (name, def_name); 
	break;	
      } 

    if (name == NULL) 
      return 1; 		/* not contain default printer section */
  
    while (!feof(file))
      {
	fgets (line, 1024, file);

	/* Seeking to the section of the printer */
	if (!strstr (line, name) || (!strstr (line, "<printer name=")))
	  continue;
	/* get information */
	while (1)
	  {	    
	    fgets (line, 1024, file );    

	    if (strstr (line , "</printer>"))
	      break;

	    value = strchr (line, '>') + 1;
	    i = strchr (value, '<') - value;
	    value[i] = '\0';	    
	    
	    if (strstr (line ,"<aliasname>" ))
	      {
		strcpy (p_info->printer_name, value);
		continue;
	      }

	    if (strstr (line , "<devicepath>"))
	      {
		strcpy (p_info->devprt_path, value);
		continue;
	      }

	    if (strstr (line , "<dummypath>"))
	      {
		strcpy (p_info->infifo_path, value);
		continue;
	      }

	    if (strstr (line , "<serverport>"))
	      {
		p_info->comsock_port = (int) atoi ((char *)value);
		continue;
	      }		   
	  }
	break;
      }


    if (*(p_info->printer_name) == 0
            || *(p_info->devprt_path) == 0
            || *(p_info->infifo_path) == 0
            || p_info->comsock_port < 0
            || p_info->comsock_port > UINT_MAX)
        return 1;    

    return 0; 
}

