/*
 * Photo Image Print System
 * Copyright (C) 2000-2005 EPSON AVASYS Corporation.
 * Copyright (C) Seiko Epson Corporation 2000-2008.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, EPSON AVASYS Corporation gives permission to
 * link the code of this program with libraries which are covered by
 * the EPSON AVASYS Public License and distribute their linked
 * combinations.  You must obey the GNU General Public License in all
 * respects for all of the code used other than the libraries which
 * are covered by EPSON AVASYS Public License.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "pipscap-util.h"

#define MAX_OPT 10

/***************************Define internal function***************************/
static void FreeContextNode (CapContextList *node);

static void
FreeContextNode (CapContextList *node)
{
    if (*node != NULL)
    {
	if ((*node)->option != NULL)
	{
	    free ((*node)->option);
	}
	free (*node);
	*node = NULL;
    }

    return;
}

/*****************************************************************************/
char*
GetValueCap (char *list, int pos)
{
    int i = 1;
    char *strTemp;
    char *value = NULL, *list_temp = NULL;

    list_temp = strdup (list);
    if (list_temp == NULL)
    {
	/* memory error */
	return NULL;
    }
    strTemp = strtok (list_temp, "/");
    while (i < pos)
    {
        strTemp = strtok (NULL, "/");
        i ++;
    }

    if (strTemp)
        value = strdup (strTemp);
    else
	value = strdup (list);

    free (list_temp);
    list_temp = NULL;

    return value;
}

/*****************************************************************************/
void
FreeValueCap (char *value)
{
    if (value != NULL) {
	free (value);
	value = NULL;
    }
    return;
}

/*****************************************************************************/
char**
GetAllValueCap (char *list)
{
    char *strTemp = NULL, *list_temp = NULL;
    char **value;
    int i = 0;

    list_temp = strdup (list);
    if (list_temp == NULL)
    {
	/* memory error */
	return NULL;
    }

    strTemp = strtok (list_temp, "/");
    value = (char **) malloc (sizeof (char *) * MAX_OPT);
    if (!strTemp)
        value[i++] = strdup (list_temp);
    else
    {
        while (strTemp)
        {
            value[i++] = strdup (strTemp);
            strTemp = strtok (NULL, "/");
        }
    }
    value[i] = NULL;

    free (list_temp);
    list_temp = NULL;

    return value;
}

/*****************************************************************************/

inline int
AddCapListCap (CapList *list, int capid, char *param)
{
    /* Create new cap node */
    CapList node = (CapList) malloc (sizeof (CapNode));
    if (!node)
        return 1;

    node->capid = capid;
    if (param)
    {
        node->param = strdup (param);
    }
    else
    {
	node->param = NULL;
    }

    node->list = NULL;
    node->next = NULL;
    /* Add to the list */
    if ((*list) == NULL)
    {
        *list = node;
    }
    else
    {
        node->next = *list;
        *list = node;
    }

    return 0;
}

/*****************************************************************************/
inline int
AddPaperSizeListCap (PaperSizeList *list, char *ptid)
{
    PaperSizeList node = (PaperSizeList) malloc (sizeof (PaperSizeNode));
    if (node == NULL)
        return 1;

    node->ptid = strdup (ptid);
    node->width = node->height = 0;
    node->next = NULL;
    /* Add to the list */
    if ((*list) == NULL)
        *list = node;
    else
    {
        node->next = *list;
        *list = node;
    }

    return 0;
}

/*****************************************************************************/
inline int
AddConsPaperSourceListCap (ConsPaperSourceList *list, char *psid)
{
    ConsPaperSourceList node = (ConsPaperSourceList) malloc (sizeof (ConsPaperSourceNode));

    if (node == NULL)
        return 1;

    node->psid = strdup (psid);
    node->list = NULL;
    node->next = NULL;
    /* Add to the list */
    if ((*list) == NULL)
        *list = node;
    else
    {
        node->next = *list;
        *list = node;
    }
    return 0;
}

/*****************************************************************************/
inline int
AddConsPaperSizeListCap (ConsPaperSizeList *list, char *ptid)
{
    ConsPaperSizeList node = (ConsPaperSizeList) malloc (sizeof (ConsPaperSizeNode));

    if (node == NULL)
        return 1;

    node->ptid = strdup (ptid);
    node->next = NULL;
    node->pa_width = node->pa_length = node->right_margin =
                     node->left_margin = node->top_margin = node->bottom_margin = 0;

    /* Add to the list */
    if ((*list) == NULL)
        *list = node;
    else
    {
        node->next = *list;
        *list = node;
    }
    return 0;
}

/*****************************************************************************/
inline int
AddPrintModeListCap (PrintModeList *list, char *mode_name)
{
    PrintModeList node = (PrintModeList) malloc (sizeof (PrintModeNode));
    if (node == NULL)
        return 1;

    node->list = NULL;
    node->mode_name = NULL;
    node->next = NULL;
    if (mode_name)
        node->mode_name = strdup (mode_name);

    /* Add to the list */
    if ((*list) == NULL)
    {
        node->modeid = 1;
        *list = node;
    }
    else
    {
        node->modeid = (*list)->modeid + 1;
        node->next = *list;
        *list = node;
    }

    return 0;
}

/*****************************************************************************/
inline int
AddElementListCap (ElementList *list, char *option, char *value)
{
    /* Create new cap node */
    ElementList node = (ElementList) malloc (sizeof (ElementNode));

    if (node == NULL)
        return 1;
//printf("AddElementListCap : %s\n", value);
    node->option = strdup (option);
    node->value = strdup (value);
    node->next = NULL;

    /* Add to the list */
    if ((*list) == NULL)
        *list = node;
    else
    { /* Checking exsit */
        if (strcmp ((*list)->option, node->option))
        {
            node->next = *list;
            *list = node;
        }
    }

    return 0;
}

/*****************************************************************************/
inline int
AddContextListCap (CapContextList *list, int capid, char *option)
{
    CapContextList curr, node;

    curr = *list;
    while (curr != NULL)
    {
        if ((curr->capid == capid))
        {
		if (strcmp (option, curr->option))
		{
			free (curr->option);
			curr->option = strdup (option);
		}
		return 0;
        }
        curr = curr->next;
    }

    /* Create new cap node */
    node = (CapContextList) malloc (sizeof (CapContextNode));
    if (node == NULL)
        return 1;

    node->capid = capid;
    node->option = strdup (option);
    node->next = NULL;
    /* Add to the list */
    if ((*list) == NULL)
        *list = node;
    else
    {
        /* Checking exsit */
        if (node->capid != (*list)->capid)
        {
            node->next = *list;
            *list = node;
        }
    }
    return 0;
}


/*****************************************************************************/
int
UpdateContextListCap (CapContextList list, int capid, char *option)
{
    CapContextList curr = list;
    while (curr != NULL)
    {
        if ((curr->capid == capid) && (strcmp (option, curr->option)))
        {
            free (curr->option);
            curr->option = strdup (option);
            return 0;
        }
        curr = curr->next;
    }
    /* Not exist capid */
    return 1;
}

/*****************************************************************************/
inline char*
SearchElementListCap (ElementList list, char *option)
{
    ElementList curr = list;
    while (curr != NULL)
    {
        if (!strcmp (curr->option, option))
            return curr->value;
        curr = curr->next;
    }

    return NULL;
}

/*****************************************************************************/
inline char*
SearchContextListCap (CapContextList list, int capid)
{
    CapContextList curr = list;
    while (curr != NULL)
    {
        if (curr->capid == capid)
            return curr->option;
        curr = curr->next;
    }

    return NULL;
}
/*****************************************************************************/
inline CapList
SearchCapListCap (CapList list, int capid)
{
    CapList curr = list;
    while ((curr != NULL) && (curr->capid != capid))
        curr = curr->next;

    return curr;
}


/*****************************************************************************/
void
DeleteContextNode (CapContextList list, int pos)
{
    CapContextList curr = list;
    int i = 0;

    if (pos < 0)
        return;
    else if (pos == 0)
    {
        list = list->next;
        FreeContextNode (&curr);
    }
    else
    {
        while (curr != NULL && i < pos-1)
        {
            curr = curr->next;
            i += 1;
        }
    }
    CapContextList node = curr->next;
    curr->next = curr->next->next;
    FreeContextNode (&node);

}

/*****************************************************************************/
void
DeleteContextListCap (CapContextList *list)
{
    CapContextList temp;
    while (*list != NULL)
    {
        temp = *list;
        *list = (*list)->next;
        FreeContextNode (&temp);
    }
    *list = NULL; 
}
/*****************************************************************************/
